/**
   A FunctionEnvironment object holds <variable,tree> pairs.

   The "tree" that we store in the <variable.tree> pair is a
   reference to the abstract syntax tree of the Lambda term
   that defines the function.
*/
import java.util.ArrayList;

public class FunctionEnvironment
{
   private ArrayList<String> variables;
   private ArrayList<Tree>   trees;


   // Constructors
   public FunctionEnvironment()
   {
      variables = new ArrayList<String>();
      trees     = new ArrayList<Tree>();
   }

   /**
      Add a <variable, tree> pair to this environment object.
   */
   public void add(String variable, Tree tree)
   {
      variables.add(variable);
          trees.add(tree);
   }

   /**
      Look up variable in the environment and
      return its associated Tree.

      Throws an Error if variable is not found.
   */
   public Tree lookUp(String variable)
   {
      int i;
      for (i=0; i < variables.size(); i++)
         if (variable.equals(variables.get(i)))
            break;

      if ( i < variables.size() )
      {
         return trees.get(i);
      }
      else
      {
         throw new Error("undefined function: " + variable);
      }
   }

   /**
      Look up variable in the environment.
      Return true if the variable is in the
      environment, otherwise return false.
   */
   public boolean defined(String variable)
   {
      int i;
      for (i=0; i < variables.size(); i++)
         if (variable.equals(variables.get(i)))
            break;

      if ( i < variables.size() )
      {
         return true;
      }
      else
      {
         return false;
      }
   }


   /**
      Convert the contents of the environment into a string.
      This is mainly for debugging purposes.
   */
   public String toString()
   {
      String result = "";

      result += "[Function Environment";

      for (int i = 0; i < variables.size(); i++)
      {
         result += "\n " + variables.get(i) + " = " + "lambda";
      }
      result += "]";

      return result;
   }

}//FunctionEnvironment